#include <bits/stdc++.h>

#define eprintf(...) fprintf(stderr, __VA_ARGS__)

class DSU {
    std::vector <int> rank, parent;
    int vertexCount;

public:
    DSU() {
        vertexCount = 0;
        rank.clear();
        parent.clear();
    }

    DSU(int vertexCount) {
        this->vertexCount = vertexCount;
        rank.resize(vertexCount);
        parent.resize(vertexCount);
        for (int i = 0; i < vertexCount; i++) {
                parent[i] = i;
                rank[i] = 0;
        }
    }

    int get(int vertex) {
        return parent[vertex] == vertex ? vertex : parent[vertex] = get(parent[vertex]);
    }

    void unite(int leftVertex, int rightVertex) {
        leftVertex = get(leftVertex);
        rightVertex = get(rightVertex);
        if (leftVertex == rightVertex) {
            return;
        }
        if (rank[leftVertex] > rank[rightVertex]) {
            std::swap(leftVertex, rightVertex);
        }
        parent[leftVertex] = rightVertex;
        if (rank[leftVertex] == rank[rightVertex]) {
            ++rank[rightVertex];
        }
    }
};

class Edge {
public:
    int v, u, w;

    Edge() : v(0), u(0), w(0) {
    }

    Edge(int v, int u, int w) : v(v), u(u), w(w) {
    }
};

bool operator < (Edge const& lhs, Edge const& rhs) {
    return lhs.w < rhs.w;
}

bool canFindCycle(int n, int m, std::vector <Edge> &edges, long long answer) {
    for (int i = 0; i < m; i++) {
        edges[i].w -= answer / 2;
    }
    int firstPositiveIndex = m;
    for (int i = 0; i < m; i++) {
        if (edges[i].w >= 0) {
            firstPositiveIndex = i;
            break;
        }
    }

    DSU dsu(n);
    int takenEdgeIndex = firstPositiveIndex;
    bool foundAnswer = false;
    for (int edgeIndex = firstPositiveIndex; edgeIndex < m; edgeIndex++) {
        int i;
        for (i = takenEdgeIndex - 1; i >= 0 && edges[i].w >= -edges[edgeIndex].w; i--) {
            dsu.unite(edges[i].v, edges[i].u);
        }
        if (dsu.get(edges[edgeIndex].v) == dsu.get(edges[edgeIndex].u)) {
            foundAnswer = true;
        }
        dsu.unite(edges[edgeIndex].v, edges[edgeIndex].u);
        takenEdgeIndex = i + 1;
    }
    for (int i = 0; i < m; i++) {
        edges[i].w += answer / 2;
    }
    return foundAnswer;
}

int main() {
    freopen("cycle.in", "r", stdin);
    freopen("cycle.out", "w", stdout);
    int n, m;
    scanf("%d%d", &n, &m);
    assert(n <= 100 && m <= 100);
    std::vector<Edge> edges;
    for (int i = 0; i < m; i++) {
        int v, u, w;
        scanf("%d%d%d", &v, &u, &w);
        --v, --u;
        edges.push_back(Edge(v, u, 2 * w));
    }
    std::sort(edges.begin(), edges.end());
    long long left = 0, right = 2e9 + 100;
    while (left < right - 1) {
        long long middle = (left + right) / 2;
        if (canFindCycle(n, m, edges, 2 * middle)) {
            left = middle;
        } else {
            right = middle;
        }
    }
    printf("%I64d\n", left);
    return 0;
}
