#include <cstdio>
#include <set>
#include <algorithm>
#include <iostream>
#include <vector>
#include <cstring>
#include <ctime>
using namespace std;
#define NAME "cycle"
const int LOGN = 17;
const int N = 1e5 + 10;
const int INF = 1e9 + 10;
struct Dsu {
    int p[N];
    int rank[N];
    Dsu() {}
    Dsu(int n) {
        for (int i = 0; i < n; ++i) {
            rank[i] = 0;
            p[i] = i;
        }
    }
    int root(int v) {
        return (p[v] == v ? v : p[v] = root(p[v]));
    }

    bool join(int x, int y) {
        x = root(x);
        y = root(y);
        if (x == y)
            return false;
        if (rank[x] > rank[y])
            swap(x, y);
        p[y] = x;
        rank[x] += rank[x] == rank[y];
        return true;
    }
};

struct Edge {
    int x, y, w;
    Edge(int x, int y, int w): x(x), y(y), w(w) {}
    bool operator < (const Edge& t) const {
        return w > t.w;
    }
};

int mx[LOGN][N];
int mn[LOGN][N];
int jump[LOGN][N];
int lev[N];
bool use[N];
vector <pair <int, int> > tree[N];

void dfs(int v) {
    use[v] = true;
    for (int h = 1; (1<<h) <= lev[v]; ++h) {
        int p = jump[h - 1][v];
        jump[h][v] = jump[h - 1][p];
        mn[h][v] = min(mn[h - 1][v], mn[h - 1][p]);
        mx[h][v] = max(mx[h - 1][v], mx[h - 1][p]);
    }

    for (size_t i = 0; i < tree[v].size(); ++i) {
        int to = tree[v][i].first;
        if (use[to]) continue;
        jump[0][to] = v;
        mn[0][to] = tree[v][i].second;
        mx[0][to] = tree[v][i].second;
        lev[to] = lev[v] + 1;
        dfs(to);
    }
}

int relax(int v, int i, int& minEdge, int& maxEdge) {
    maxEdge = max(maxEdge, mx[i][v]);
    minEdge = min(minEdge, mn[i][v]);
    return jump[i][v];
}

int up(int v, int diff, int& minEdge, int& maxEdge) {
    for (int i = 0; diff; i++, diff >>= 1)
        if (diff&1)
            v = relax(v, i, minEdge, maxEdge);
    return v;
}
pair <int, int> findMaxMin(int x, int y) {
    if (lev[x] < lev[y])
        swap(x, y);
    int mx = 0, mn = INF;
    x = up(x, lev[x] - lev[y], mn, mx);
    if (x == y)
        return make_pair(mn, mx);
    for (int i = LOGN - 1; i >= 0; --i)
        if (jump[i][x] != jump[i][y]) {
            x = relax(x, i, mn, mx);
            y = relax(y, i, mn, mx);
        }
    relax(x, 0, mn, mx);
    relax(y, 0, mn, mx);
    return make_pair(mn, mx);
}

int n, m;
vector <Edge> e;
Dsu s;
bool useEdge[N];

int main() {
    freopen(NAME".in", "r", stdin);
    freopen(NAME".out", "w", stdout);
    cin >> n >> m;
    for (int i = 0; i < m; ++i) {
        int x, y, w;
        scanf("%d%d%d", &x, &y, &w);
        e.push_back(Edge(x, y, w));
    }
    sort(e.begin(), e.end());
    s = Dsu(n);
    for (int i = 0; i < m; ++i)
        if (s.join(e[i].x, e[i].y)) {
            useEdge[i] = true;
            tree[e[i].x].push_back(make_pair(e[i].y, e[i].w));
            tree[e[i].y].push_back(make_pair(e[i].x, e[i].w));
        }
    for (int i = 0; i < n; ++i)
        if (!use[i]) dfs(i);
    int ans = 0;
    for (int i = 0; i < m; ++i)
        if (!useEdge[i]) {
            pair <int, int> res = findMaxMin(e[i].x, e[i].y);
            res.first = min(res.first, e[i].w);
            res.second = max(res.second, e[i].w);
            ans = max(ans, res.first + res.second);
        }
    cout << ans;
    return 0;
}
