import java.io.*;
import java.util.*;

public class TestGen {

    static PrintWriter out;
    static Random rnd = new Random(0);
    static int tNumber = 1;
    static PrintWriter testDescr;
    static int sb = 0;


    static void printTest(int n, int m, int k, Query[] q, String comment) throws IOException {
        System.out.println(String.format("Generating test %d", tNumber));
        PrintWriter commonOut;
        if (tNumber < 10) 
            out = new PrintWriter("../tests/0" + tNumber);
        else
            out = new PrintWriter("../tests/" + tNumber);
        if (tNumber+nmbSamples < 10) 
            commonOut = new PrintWriter("../common/0" + (tNumber+nmbSamples));
        else
            commonOut = new PrintWriter("../common/" + (tNumber+nmbSamples));
        
        out.println(n + " " + m + " " + k);
        commonOut.println(n + " " + m + " " + k);
        
        for (int i = 0; i < m; i++) {
            out.println((q[i].l+1) + " " + (q[i].r+1) + " " + q[i].x);    
            commonOut.println((q[i].l+1) + " " + (q[i].r+1) + " " + q[i].x);    
        }
        out.close();
        commonOut.close();
        testDescr.println("../tests/" + ((tNumber > 9) ? tNumber : ("0" + tNumber)) + ", folder=tests, subtask=subtask" + sb + ", points=0, comment=\"" + comment + "\"");
        tNumber++;        
    }
    
    static int nmbSamples = 1;

    static void genSample() throws FileNotFoundException {
        out = new PrintWriter("../preliminary/01");        
        out.println("5 3 5");
        out.println("1 3 2");
        out.println("1 2 1");
        out.println("1 5 5");
        out.close();

        out = new PrintWriter("../common/01");        
        out.println("5 3 5");
        out.println("1 3 2");
        out.println("1 2 1");
        out.println("1 5 5");

        out.close();                
        testDescr.println("../preliminary/01, folder=preliminary, subtask=subtask0, points=0, comment=\"sample 1\"");
        
    }
    
    static int randFromTo(int l, int r) {
        return rnd.nextInt(r - l + 1) + l;      
    }

    static Query[] randomQueries(int n, int m, int k) {
        Query[] q = new Query[m];
        for (int i = 0; i < m; i++) {
            int r = randFromTo(0, n-1);
            int l = randFromTo(0, r);
            int x = randFromTo(1, k);
            q[i] = new Query(l, r, x);
        }
        return q;
    }

    static Query[] realQueries(int n, int m, int k) {
        int[] array = new int[n];
        int[][] rmq = new int[20][n];
        int[] pow = new int[n + 1];
        for (int i = 0; i <= n; i++) {
            int p = 0;
            while ((1 << p) <= i)++p;
            pow[i] = p-1;
        }
        for (int i = 0; i < n; i++) {
            array[i] = randFromTo(1, k);
            rmq[0][i] = array[i];
        }
        for (int i = 1; i < 20; i++)
            for (int j = 0; j + (1 << i) - 1 < n; ++j)
                rmq[i][j] = Math.max(rmq[i-1][j], rmq[i-1][j + (1<<(i-1))]);
        Query[] q = new Query[m];
        for (int i = 0; i < m; i++) {
            int r = randFromTo(0, n-1);
            int l = randFromTo(0, r);
            int p = pow[r-l+1];
            int x = Math.max(rmq[p][l], rmq[p][r - (1 << p) + 1]);
            q[i] = new Query(l, r, x);
        }
        return q;
    }

	static Query[] realShortQueries(int n, int m, int k, int len) {
        int[] array = new int[n];
        int[][] rmq = new int[20][n];
        int[] pow = new int[n + 1];
        for (int i = 0; i <= n; i++) {
            int p = 0;
            while ((1 << p) <= i)++p;
            pow[i] = p-1;
        }
        for (int i = 0; i < n; i++) {
            array[i] = randFromTo(1, k);
            rmq[0][i] = array[i];
        }
        for (int i = 1; i < 20; i++)
            for (int j = 0; j + (1 << i) - 1 < n; ++j)
                rmq[i][j] = Math.max(rmq[i-1][j], rmq[i-1][j + (1<<(i-1))]);
        Query[] q = new Query[m];
        for (int i = 0; i < m; i++) {
            int r = randFromTo(0, n-1);
            int l = randFromTo(Math.max(0, r - len), r);
            int p = pow[r-l+1];
            int x = Math.max(rmq[p][l], rmq[p][r - (1 << p) + 1]);
            q[i] = new Query(l, r, x);
        }
        return q;
    }

	static Query[] realDifferentQueries(int n, int m, int k) {
        int[] array = new int[n];
        int[][] rmq = new int[20][n];
        int[] pow = new int[n + 1];
        for (int i = 0; i <= n; i++) {
            int p = 0;
            while ((1 << p) <= i)++p;
            pow[i] = p-1;
        }
        for (int i = 0; i < n; i++) {
            array[i] = randFromTo(1, k);
            rmq[0][i] = array[i];
        }
        for (int i = 1; i < 20; i++)
            for (int j = 0; j + (1 << i) - 1 < n; ++j)
                rmq[i][j] = Math.max(rmq[i-1][j], rmq[i-1][j + (1<<(i-1))]);
        Query[] q = new Query[m];
        int len = 1;
        Set<Integer> was = new TreeSet<Integer>();
        int c = 0;
        for (int i = 0; i < m; i++) {
            int r = randFromTo(0, n-1);
            int l = randFromTo(Math.max(0, r - len), r);
            int p = pow[r-l+1];
            int x = Math.max(rmq[p][l], rmq[p][r - (1 << p) + 1]);
            if (!was.contains(x)) {
	            q[c++] = new Query(l, r, x);
	            was.add(x);
	        }
            len += (n / m) + 1;
        }
        Query[] q1 = new Query[c];
        System.arraycopy(q, 0, q1, 0, c);
        return q1;
    }




    public static void main(String[] args) throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();        
        testDescr = new PrintWriter("../tests.desc");

        genSample();
        {
            sb = 1;
            {
                int n = 1;
                int m = 10;
                int k = 50;
                Query[] q = randomQueries(n, m, k);
                printTest(n, m, k, q, "min test: n = " + n + ", m = " + m + ", k = " + k);    
            }
            {
                int n = 1;
                int m = 0;
                int k = 50;
                Query[] q = randomQueries(n, m, k);
                printTest(n, m, k, q, "min test: n = " + n + ", m = " + m + ", k = " + k);    
            }    
            for (int i = 0; i < 10; i++) {
                int n = randFromTo(20, 50);
                int m = 10;
                int k = 50;
                Query[] q = realQueries(n, m, k);
                printTest(n, m, k, q, "small random test(real query): n = " + n + ", m = " + m + ", k = " + k);
            }
            for (int i = 0; i < 5; i++) {
                int n = randFromTo(20, 50);
                int m = 10;
                int k = 5;
                Query[] q = randomQueries(n, m, k);
                printTest(n, m, k, q, "small random test: n = " + n + ", m = " + m + ", k = " + k);
            }

			for (int i = 0; i < 5; i++) {
                int n = randFromTo(20, 50);
                int m = 10;
                int k = 5;
                Query[] q = realShortQueries(n, m, k, 5);
                printTest(n, m, k, q, "small random test: n = " + n + ", m = " + m + ", k = " + k);
            }

            {
                int n = 50;
                int m = 10;
                int k = 50;
                Query[] q = randomQueries(n, m, k);
                printTest(n, m, k, q, "max test: n = " + n + ", m = " + m + ", k = " + k);    
            }    
            {
                int n = 50;
                int m = 10;
                int k = 50;
                Query[] q = randomQueries(n, m, k);
                q[0] = new Query(1, 10, 0);
                q[0] = new Query(1, 10, 10);
                printTest(n, m, k, q, "0 test: n = " + n + ", m = " + m + ", k = " + k);    
            }    
            
            testDescr.println("comment_subtask1=\"n &lt;= 50, m &lt;= 10, k &lt;= 50\"\nbonus_subtask1=30");
        }

        {
            sb = 2;
            for (int i = 0; i < 10; ++i) {
                int n = randFromTo(10000, 100000);
                int m = randFromTo(10000, 100000);
                int k = randFromTo(100000, 1000000000);
                Query[] q = realDifferentQueries(n, m, k);
               	printTest(n, q.length, k, q, "diff queries test: n = " + n + ", m = " + m + ", k = " + k);                
           	} 
           	{
           	    int n = randFromTo(10000, 100000);
                int m = n;
                int k = n + 10;
            	Query[] q = new Query[m];
            	for (int i = 0; i < m; ++i) {
					q[i] = new Query(i, i, i+1);            
            	}
                printTest(n, q.length, k, q, "diff queries test: n = " + n + ", m = " + m + ", k = " + k);                           
           	}            
            testDescr.println("comment_subtask2=\"n &lt;= 100000, m &lt;= 100000, k &lt;= 1000000000, different queries\"\nbonus_subtask2=30");
        }
        {
            sb = 3;
            {
                int n = 1;
                int m = 1000;
                int k = 1000000000;
                Query[] q = randomQueries(n, m, k);
                printTest(n, m, k, q, "min test: n = " + n + ", m = " + m + ", k = " + k);    
            }
            {
                int n = 1;
                int m = 0;
                int k = 1000000000;
                Query[] q = randomQueries(n, m, k);
                printTest(n, m, k, q, "min test: n = " + n + ", m = " + m + ", k = " + k);    
            }    
            for (int i = 0; i < 5; i++) {
                int n = randFromTo(100, 1000);
                int m = 1000;
                int k = randFromTo(10, 100);
                Query[] q = realQueries(n, m, k);
                printTest(n, m, k, q, "big random test(real): n = " + n + ", m = " + m + ", k = " + k);
            }
            for (int i = 0; i < 5; i++) {
                int n = randFromTo(1000, 10000);
                int m = 1000;
                int k = randFromTo(10, 100);
                Query[] q = realQueries(n, m, k);
                printTest(n, m, k, q, "big random test(real query): n = " + n + ", m = " + m + ", k = " + k);
            }
            for (int i = 0; i < 5; i++) {
                int n = randFromTo(10000, 100000);
                int m = randFromTo(10000, 100000);
                int k = randFromTo(100000, 1000000000);
                Query[] q = realQueries(n, m, k);
                printTest(n, m, k, q, "big random test(real): n = " + n + ", m = " + m + ", k = " + k);
            }
            for (int i = 0; i < 5; i++) {
                int n = randFromTo(10000, 100000);
                int m = randFromTo(10000, 100000);
                int k = randFromTo(100000, 1000000000);
                Query[] q = realQueries(n, m, k);
                printTest(n, m, k, q, "big random test(real query): n = " + n + ", m = " + m + ", k = " + k);
            }


            for (int i = 0; i < 2; i++) {
                int n = randFromTo(10000, 100000);
                int m = randFromTo(10000, 100000);
                int k = randFromTo(100000, 1000000000);
                Query[] q = randomQueries(n, m, k);
                printTest(n, m, k, q, "big random test(real query): n = " + n + ", m = " + m + ", k = " + k);
            }
            
            for (int i = 0; i < 2; i++) {
                int n = randFromTo(10000, 100000);
                int m = randFromTo(10, 100);
                int k = randFromTo(100000, 1000000000);
                Query[] q = randomQueries(n, m, k);
                printTest(n, m, k, q, "big random test(real query): n = " + n + ", m = " + m + ", k = " + k);
            }
            for (int i = 0; i < 5; i++) {
                int n = randFromTo(10000, 100000);
                int m = randFromTo(10000, 100000);
                int k = randFromTo(10, 100);
                Query[] q = realQueries(n, m, k);
                printTest(n, m, k, q, "big random test(real query): n = " + n + ", m = " + m + ", k = " + k);
            }

			for (int i = 0; i < 5; i++) {
                int n = randFromTo(10000, 100000);
                int m = randFromTo(10000, 100000);
                int k = randFromTo(100000, 1000000000);
                Query[] q = realShortQueries(n, m, k, (int)Math.sqrt(n));
                printTest(n, m, k, q, "big random test(real short query): n = " + n + ", m = " + m + ", k = " + k);
            }


            {
                int n = 100000;                       
                int m = 100000;
                int k = 1000000000;
                Query[] q = new Query[m];
                for (int i = 0; i < q.length; i++) {
                    q[i] = new Query();
                    q[i].l = randFromTo(1, 10000);
                    q[i].r = randFromTo(90000, 99999);
                    q[i].x = 10;
                }
                printTest(n, m, k, q, "max test: n = " + n + ", m = " + m + ", k = " + k);        
            }
            {
                int n = 100000;
                int m = 100000;
                int k = 1000000000;
                Query[] q = randomQueries(n, m, k);
                printTest(n, m, k, q, "max test: n = " + n + ", m = " + m + ", k = " + k);    
            }    
            {
                int n = 100000;
                int m = 100000;
                int k = 1000000000;
                Query[] q = randomQueries(n, m, k);
                q[0] = new Query(123, 1230, 0);
                q[0] = new Query(123, 1230, 10);
                printTest(n, m, k, q, "0 test: n = " + n + ", m = " + m + ", k = " + k);    
            }    
            
            testDescr.println("comment_subtask3=\"n &lt;= 100000, m &lt;= 100000, k &lt;= 1000000000\"\nbonus_subtask2=40");
        }
        testDescr.close();
    }                 

    private static class Query {
        int l, r, x;
        Query(int l, int r, int x) {
            this.l = l;
            this.r = r;
            this.x = x;
        }
        Query(){}
    }
    
}
