/**
 * Created by pva701 on 12.03.2015.
 */

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import ru.ifmo.testlib.*;
import ru.ifmo.testlib.Outcome.Type;

public class Check implements Checker {

    public static int MAX_PRIME = 10000000;
    private static boolean[] p = new boolean[MAX_PRIME + 1];
    private boolean[] use = new boolean[MAX_PRIME + 1];
    private ArrayList<Pair<Integer, Integer>>[] factorization;

    public static class Pair<K, V> {
        private K key;
        private V value;
        Pair(K k, V v) {
            key = k;
            value = v;
        }

        public K getKey() {
            return key;
        }

        public V getValue() {
            return value;
        }
    }
    @Override
    public Outcome test(InStream inf, InStream ouf, InStream ans) {
        long lt = System.currentTimeMillis();

        int k = inf.nextInt();
        int n = inf.nextInt();
        int juryAns = ans.nextInt();
        int partAns = ouf.nextInt();
        if (juryAns == -1) {
            if (partAns != -1) {
                return new Outcome(Type.WA, "Jury haven't answer, but participant have answer!");
            }
            if (!ouf.seekEoF()) {
                return new Outcome(Type.PE, "Too much tokens in answer");
            }
            return new Outcome(Type.OK, "Ok. No solution.");
        }
        int[][] a = new int[n][n];
        int[][] b = new int[n][n];
        a[0][0] = partAns;
        b[0][0] = partAns;
        for (int i = 0; i < n; ++i) {
            for (int j = (i == 0 ? 1 : 0); j < n; ++j)
                b[j][i] = a[i][j] = ouf.nextInt();
            ouf.nextLine();
        }

        if (!ouf.seekEoF()) {
            return new Outcome(Type.PE, "Too much tokens in answer");
        }

        buildSieve();
        ArrayList<Integer> pr = new ArrayList<Integer>();
        factorization = new ArrayList[n + 2];
        for (int i = 2; i <= n + 1; ++i)
            if (ensurePrime(i)) {
                factorization[i] = new ArrayList<Pair<Integer, Integer>>();
                factorization[i].add(new Pair<Integer, Integer>(i, 1));
                pr.add(i);
            } else {
                int p = 0;
                for (int j = 0; j < pr.size() && pr.get(j) * pr.get(j) <= i; ++j)
                    if (i % pr.get(j) == 0) {
                        p = pr.get(j);
                        break;
                    }
                factorization[i] = new ArrayList<Pair<Integer, Integer>>();
                int prev = i / p;
                boolean found = false;
                for (int j = 0; j < factorization[prev].size(); ++j) {
                    Pair<Integer, Integer> pair = factorization[prev].get(j);
                    if (pair.getKey() == p) {
                        factorization[i].add(new Pair<Integer, Integer>(p, pair.getValue() + 1));
                        found = true;
                    } else
                        factorization[i].add(pair);
                }
                if (!found)
                    factorization[i].add(new Pair<Integer, Integer>(p, 1));
            }

        int diffs = 0;
        for (int i = 0; i < n; ++i)
            for (int j = 0; j < n; ++j) {
                int x = a[i][j];
                if (!ensurePrime(x))
                    return new Outcome(Type.WA, "a[" + i + "][" + j + "] element isn't prime or not in [1;" + MAX_PRIME + "]");
                if (!use[x]) {
                    ++diffs;
                    use[x] = true;
                }
            }
        if (diffs > k)
            return new Outcome(Type.WA, "Participant use more than " + k + " prime numbers!");
        if (diffs < k)
            return new Outcome(Type.WA, "Participant use less than " + k + " prime numbers!");

        int[] arr1 = checkMatrix(a);
        if (arr1 == null)
            return new Outcome(Type.WA, "Different numbers of divisors!");

        int[] arr2 = checkMatrix(b);
        if (arr2 == null || !Arrays.equals(arr1, arr2))
            return new Outcome(Type.WA, "Different numbers of divisors!");

        //System.err.println("TIME CHECKER = " + (System.currentTimeMillis() - lt) / 1000.0);
        return new Outcome(Type.OK, "Ok. Correct solution!");
    }

    private int[] checkMatrix(int[][] a) {
        int n = a.length;
        int[] divisors = null;
        int[] iter = new int[MAX_PRIME + 1];
        int[] cnt = new int[MAX_PRIME + 1];
        for (int i = 0; i < n; ++i) {
            ArrayList <Integer> cnts = new ArrayList<Integer>();
            ArrayList <Integer> pr = new ArrayList<Integer>();
            for (int j = 0; j < n; ++j) {
                int x = a[i][j];
                if (iter[x] != i + 1) {
                    iter[x] = i + 1;
                    cnt[x]  = 0;
                    pr.add(x);
                }
                ++cnt[x];
            }
            for (int j = 0; j < pr.size(); ++j)
                cnts.add(cnt[pr.get(j)] + 1);
            int[] c1 = countDivisors(n + 1, cnts);
            if (divisors == null)
                divisors = c1;
            if (!Arrays.equals(divisors, c1))
                return null;
        }
        return divisors;
    }

    private static boolean ensurePrime(int x) {
        return 1 < x && x <= MAX_PRIME && !p[x];
    }

    private int[] countDivisors(int maxPr, ArrayList <Integer> cnts) {
        int[] ret = new int[maxPr + 1];
        for (Integer x : cnts) {
            ArrayList <Pair<Integer, Integer> > fact = factorization[x];
            for (int j = 0; j < fact.size(); ++j)
                ret[fact.get(j).getKey()] += fact.get(j).getValue();
        }
        return ret;
    }

    private void addNumber(int p, Map<Integer, Integer> primes) {
        if (!primes.containsKey(p))
            primes.put(p, 0);
        primes.put(p, primes.get(p) + 1);
    }

    private static void buildSieve() {
        for (int i = 2; i * i <= MAX_PRIME; ++i)
            if (!p[i])
                for (int j = i * i; j <= MAX_PRIME; j += i)
                    p[j] = true;
    }
}
